@@ -46,6 +46,12 @@ inline T universal_pointer_cast(const std::shared_ptr<U>& sp){
 	return std::static_pointer_cast<typename T::element_type>(sp);
 }
 
+
+//template <class T, class U>
+//inline std::shared_ptr<T> universal_pointer_cast(const std::shared_ptr<U>& sp){
+//	return std::static_pointer_cast<T>(sp);
+//}
+
 template <class T, class U>
 inline T universal_pointer_cast(U * p){
 	return static_cast<T>(p);
@@ -67,18 +73,34 @@ private:
 	inline void _invokeWithValue(_parserElementT obj, typename std::enable_if<std::is_convertible<_valueT, _parserElementT>::value, const std::string&>::type value){
 		// no op.
 	}
+	#if defined(_MSC_VER)
+	#pragma warning( push )
+	#pragma warning( disable : 4244 )
+	#endif
 	template <typename _valueT>
-	inline void _invokeWithValue(_parserElementT obj, typename std::enable_if<std::is_convertible<_valueT, int>::value, const std::string&>::type value){
-		mFunc(universal_pointer_cast<typename _functorT::first_argument_type>(obj), std::atoi(value.c_str()));
+	inline void _invokeWithValue(_parserElementT obj, typename std::enable_if<std::is_integral<_valueT>::value, const std::string&>::type value){
+		mFunc(universal_pointer_cast<typename _functorT::first_argument_type>(obj), std::atoll(value.c_str()));
+	}
+	#if defined(_MSC_VER)
+	#pragma warning( pop )
+	#endif
+	template <typename _valueT>
+	inline void _invokeWithValue(_parserElementT obj, typename std::enable_if<std::is_floating_point<_valueT>::value, const std::string&>::type value){
+		mFunc(universal_pointer_cast<typename _functorT::first_argument_type>(obj), std::atof(value.c_str()));
 	}
 	template <typename _valueT>
 	inline void _invokeWithChild(_parserElementT obj, typename std::enable_if<std::is_convertible<_valueT, std::string>::value, _parserElementT>::type child){
 	}
 	template <typename _valueT>
-	inline void _invokeWithChild(_parserElementT obj, typename std::enable_if<std::is_convertible<_valueT, int>::value, _parserElementT>::type child){
+	inline void _invokeWithChild(_parserElementT obj, typename std::enable_if<std::is_integral<_valueT>::value, _parserElementT>::type child){
 	}
 	template <typename _valueT>
-	inline void _invokeWithChild(_parserElementT obj, typename std::enable_if<std::is_convertible<_valueT, _parserElementT>::value, _parserElementT>::type child){
+	inline void _invokeWithChild(_parserElementT obj, typename std::enable_if<std::is_floating_point<_valueT>::value, _parserElementT>::type child){
+	}
+	template <typename _valueT>
+	inline void _invokeWithChild(
+		typename std::enable_if< std::is_convertible<typename _functorT::first_argument_type, _parserElementT>::value, _parserElementT>::type obj, 
+		typename std::enable_if< std::is_convertible<_valueT, _parserElementT>::value, _parserElementT>::type child ){
 		mFunc(universal_pointer_cast<typename std::remove_reference<typename _functorT::first_argument_type>::type>(obj), 
 			universal_pointer_cast<typename std::remove_reference<typename _functorT::second_argument_type>::type>(child));
 	}
@@ -119,6 +141,7 @@ private:
 	std::shared_ptr<HandlerContext<_parserElementT>> mCachedContext;
 };
 
+
 template <typename _createElementFn, typename _parserElementT>
 class ParserHandler :  public ParserHandlerBase<_parserElementT>{
 public:
@@ -140,7 +163,6 @@ private:
 	}
 	template <typename _funcT>
 	typename std::enable_if<std::is_convertible<_funcT, std::function<_derivedParserElementT()>>::value, _derivedParserElementT>::type _invoke(const std::string &value, size_t begin, size_t count){
-		// Case where the create func accepts two strings for rulename and matched characters.
 		return mHandlerCreateFunc();
 	}
 	_createElementFn mHandlerCreateFunc;
@@ -286,11 +308,11 @@ public:
 	BELR_PUBLIC std::shared_ptr<DebugElement> parseInput(const std::string &rulename, const std::string &input, size_t *parsed_size);
 };
 
-//Utility functions for handlers/collectors objects instantiation and properties accessors
-template <typename _retT>
-inline std::function< std::shared_ptr<_retT> ()> make_fn() {
-	return std::bind(&std::make_shared<_retT>);
-}
+//
+// Utility functions for handlers/collectors objects instantiation and properties accessors
+//
+
+/* For parser relying on raw pointers */
 
 template <typename _retT>
 inline std::function< _retT ()> make_fn(_retT (*arg)()){
@@ -302,11 +324,33 @@ inline std::function< _retT (_arg1T,_arg2T)> make_fn(_retT (*arg)(_arg1T,_arg2T)
 	return std::function< _retT (_arg1T,_arg2T)>(arg);
 }
 
+template <typename _retT, typename _arg1T>
+struct StringToCharMapper{
+	StringToCharMapper(const std::function< _retT (_arg1T, const char*)> &cfunc) : mCFunc(cfunc){
+	}
+	std::function< _retT (_arg1T, const char*)> mCFunc; 
+	_retT operator()(_arg1T arg1, const std::string & arg2){
+		return mCFunc(arg1, arg2.c_str());
+	}
+};
+
+template <typename _retT, typename _arg1T>
+inline std::function< _retT (_arg1T, const std::string &)> make_fn(_retT (*arg)(_arg1T, const char*)){
+	return StringToCharMapper<_retT, _arg1T>(arg);
+}
+
 template <typename _klassT, typename _argT>
 inline std::function< void (_klassT*,_argT)> make_fn(void (_klassT::*arg)(_argT)){
 	return std::function< void (_klassT*,_argT)>(std::mem_fn(arg));
 }
 
+/* For parsers using shared_ptr<> */
+
+template <typename _retT>
+inline std::function< std::shared_ptr<_retT> ()> make_fn() {
+	return std::bind(&std::make_shared<_retT>);
+}
+
 template <typename _klassT, typename _argT>
 inline std::function< void (std::shared_ptr<_klassT>,_argT)> make_sfn(void (_klassT::*arg)(_argT)){
 	return std::function< void (std::shared_ptr<_klassT>,_argT)>(std::mem_fn(arg));
@@ -465,7 +509,7 @@ inline void ParserContext<_parserElementT>::_beginParse(ParserLocalContext & lct
 	if (mHandlerStack.empty()){
 		fatal("Cannot parse when mHandlerStack is empty. You must define a top-level rule handler.");
 	}
-	lctx.set(ctx,rec,mHandlerStack.back()->getLastIterator());
+	lctx.set(ctx, rec, mHandlerStack.back()->getLastIterator());
 }
 
 template <typename _parserElementT>
@@ -593,14 +637,21 @@ template <typename _parserElementT>
 _parserElementT Parser<_parserElementT>::parseInput(const std::string &rulename, const std::string &input, size_t *parsed_size){
 	size_t parsed;
 	std::shared_ptr<Recognizer> rec=mGrammar->getRule(rulename);
-	auto pctx=std::make_shared<ParserContext<_parserElementT>>(*this);
+	ParserContext<_parserElementT> pctx(*this);
+	
+	auto h=getHandler(rec->getId());
+	if (!h){
+		std::ostringstream str;
+		str<<"There is no handler for rule '"<<rulename<<"'.";
+		fatal(str.str().c_str());
+	}
 
 	//auto t_start = std::chrono::high_resolution_clock::now();
 	parsed=rec->feed(pctx, input, 0);
 	//auto t_end = std::chrono::high_resolution_clock::now();
 	//cout<<"Recognition done in "<<std::chrono::duration<double, std::milli>(t_end-t_start).count()<<" milliseconds"<<std::endl;
 	if (parsed_size) *parsed_size=parsed;
-	auto ret= pctx->createRootObject(input, parsed);
+	auto ret= pctx.createRootObject(input, parsed);
 	return ret;
 }
 }
